/**
 * Using this template, your task is to implement functions denoted by TODO in the comments.
 * 
 * In this task, you will work with directed graph given by an adjacency matrix.
 * Your task is to implement three different graph algorithms, see their description
 * by the methods of class Graph.
 * 
 * The Java project also contains tests, in Eclipse you can run them by left-clicking:
 * test/(default package)/JUnitTest.java and selecting Run as -> JUnitTest.java.
 * 
 * The tests will show you if they passed/failed. In case of fail, you can see an exception
 * if the code did not finished, or the difference between your output and expected output.
 * The test names should help you to explain what is being tested. You can also see the content
 * of the tests, for the format of the input, see <code>read</code> method.
 * 
 * The output format is described in the comment of <code>output</code> method.
 */

/**
 * This import policy applies to all programming assignments and the exam.
 * 
 * You are forbidden to add any other import statements. Make sure that you do not leave any
 * imports commented out, or that your IDE does not add any automatically. You can recognize
 * allowed imports by the <code>// allowed import</code> comment.
 * Calls by the fully qualified name are treated as imports as well.
 * You can use data structures we provide to you by imports, by our definitions (even
 * implicitly used classes of packages, such as methods of Object those are inherited with
 * every class declaration, Array, or String), or data structures that you write on your own.
 * Usage of common arrays (<code>type[] variable</code>) is not restricted.
 * 
 * Note that Judge is not enforcing this policy. So if you violate this policy, you may lose the
 * points even if Judge accepts your submission.
 * 
 * You may use the following imports without restrictions.
 */
import java.util.List; // allowed import
import java.util.LinkedList; // allowed import
import java.util.ArrayList; // allowed import
import java.util.Queue; // allowed import
import java.util.Stack; // allowed import
import java.util.Arrays; // allowed import
import java.util.StringTokenizer; // allowed import
import java.io.BufferedReader; // allowed import
import java.io.IOException; // allowed import
import java.io.InputStream; // allowed import
import java.io.InputStreamReader; // allowed import
import java.io.PrintStream; // allowed import
import java.lang.Math; // allowed import
import java.lang.Integer; // allowed import

class Main {
	public static void main(String[] args) {
		ReadAndWrite rw = new ReadAndWrite();
		rw.readAndSolve(System.in, System.out);
	}
}

class DiGraph {
	boolean[][] diGraph; // directed graph represented by an adjacency matrix
	
	// feel free to add any other attributes

	/**
	 * The class is instantiated once for every graph.
	 */
	DiGraph(boolean[][] g) {
		diGraph = g;
	}
	
	/**
	 * TODO: tests, if the the graph describes an antisymmetric relation.
	 * 
	 * Solution in O(|V|^2) gets 4 points
	 * 
	 * @return true for antisymmetric relations, else false
	 */
	boolean isAntisymmetricRelation() {
		return false;
	}
	
	/**
	 * TODO: tests, if the the graph describes a transitive relation.
	 * 
	 * Solution in O(|V|^3) gets 4 points
	 * 
	 * @return true for transitive relation, else false
	 */
	boolean isTransitiveRelation() {
		return false;
	}
	
	/**
	 * TODO: tests, if the the graph contains a directed cycle
	 * 
	 * 10 points
	 * 
	 * Solution of a set of particularly simple instances gets 2 points
	 * General solution in O(|V|^3) gets 2 points
	 * General solution in O(|V|^2) gets all 10 points
	 * 
	 * @return true for graph with cycle, false for acyclic graph
	 */
	boolean containsCycle() {
		return false;
	}
	
	/**
	 * This function is not tested by Judge or JUnitTest, it serves
	 * only for debugging - visualization by testSingle.
	 * 
	 * You can leave it as it is, but if you need to highlight
	 * some property for debugging, you can select custom color
	 * for some type of vertices.
	 * 
	 * @return string representation of color for given vertex
	 * <code>i</code>.
	 */
	String getNodeColor(int i) {
		return "white";
	}
}

class Edge {
	// a directed edge
	int u; // from vertex u
	int v; // to vertex v
	
	public Edge(int uv, int vv) {
		u = uv;
		v = vv;
	}
}

class StarDiGraph {
	int size; // the number of vertices of the graph
	Edge[] edges; // sorted list of edges

	// feel free to add any other attributes
	
	public StarDiGraph(int s, Edge[] e) {
		size = s;
		edges = e;
	}
	
	/**
	 * TODO: tests, if the the graph contains a node, which has in degree |V|-1,
	 * out degree 0, and no self loop.
	 * Such a node was called "star" in the lecture.
	 * 
	 * Solution in O(|V|*|E|) gets 2 points
	 *             O(|E|)     gets 6 points
	 * 
	 * @return true if graph contains a star, else false
	 */
	boolean containsStar() {
		return false;
	}
}

///////////////////////////////////////////////////////////////////////
// DO NOT MODIFY THE FOLLOWING CODE, YOU CAN COMPLETELY IGNORE IT
// WE MAY USE LANGUAGE CONSTRUCTS THAT YOU MAY HAVE NOT SEEN SO FAR
///////////////////////////////////////////////////////////////////////

class ReadAndWrite {
	/**
	 * The graph can have 3 different encoding depending on sparsity
	 * This encoding is determined by character 'm', 'l', or 'c'
	 * 
	 * What function will be tested is determined by character
	 * 'a' (antisymmetric), 'c' (cycle), 't' (transitive), or 's' (star)
	 */
	void readAndSolve(InputStream in, PrintStream out) {
		FastReader s = new FastReader(in);

		char testMode = s.nextChar();
		int n = s.nextInt();
		
		if (testMode == 's') {
			for (int i = 0; i < n; i++) {
				char inMode = s.nextChar();
				if (inMode != 'l') {
					System.err.println("Wrong mode!");
				}
				int size = s.nextInt();
				int nEdges = s.nextInt();
				Edge[] edges = new Edge[nEdges];
				for (int j = 0; j < nEdges; j++) {
					edges[j] = new Edge(s.nextInt(), s.nextInt());
				}

				StarDiGraph g = new StarDiGraph(size, edges);
				out.println(g.containsStar());
			}
		} else {
			for (int i = 0; i < n; i++) {
				char inMode = s.nextChar();
				int vertices = s.nextInt();
				boolean[][] matrix = new boolean[vertices][vertices];
				
				switch (inMode) {
				case 'm':
					for (int u = 0; u < vertices; u++) {
						for (int v = 0; v < vertices; v++) {
							matrix[u][v] = s.nextInt() == 1;
						}
					}
					break;
				case 'l':
					int edges = s.nextInt();
					for (int j = 0; j < edges; j++) {
						matrix[s.nextInt()][s.nextInt()] = true;
					}
					break;
				case 'c':
					for (int u = 0; u < vertices; u++) {
						for (int v = 0; v < vertices; v++) {
							matrix[u][v] = true;
						}
					}
					break;
	
				default:
					break;
				}
				
				DiGraph g = new DiGraph(matrix);
				
				switch (testMode) {
				case 'a':
					out.println(g.isAntisymmetricRelation());
					break;
				case 't':
					out.println(g.isTransitiveRelation());
					break;
				case 'c':
					out.println(g.containsCycle());
					break;
	
				default:
					break;
				}
			}
		}
	}
}

/**
 * Ignore this class please. It is used for input parsing. Source:
 * https://www.geeksforgeeks.org/fast-io-in-java-in-competitive-programming/
 */
class FastReader {
	BufferedReader br;
	StringTokenizer st;

	FastReader(InputStream in) {
		br = new BufferedReader(new InputStreamReader(in));
	}

	String next() {
		while (st == null || !st.hasMoreElements()) {
			try {
				st = new StringTokenizer(br.readLine());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return st.nextToken();
	}

	int nextInt() {
		return Integer.parseInt(next());
	}

	char nextChar() {
		return next().charAt(0);
	}

	String nextLine() {
		String str = "";
		try {
			str = br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return str;
	}
}
