/**
 * Using this template, your task is to implement functions denoted by TODO in the comments.
 * 
 * In this task, you will work strings. You are given a string <code>str</code> and an array
 * of words <code>dictionary</code>. Your task is to find, how long part of <code>str</code>
 * from position 0 can be decomposed into words in <code>dictionary</code>.
 * 
 * See the description of the task, together with notes on Java, by method solve.
 * 
 * The Java project also contains tests, in Eclipse you can run them by left-clicking:
 * test/(default package)/JUnitTest.java and selecting Run as -> JUnitTest.java.
 * 
 * The tests will show you if they passed/failed. In case of fail, you can see an exception
 * if the code did not finished, or the difference between your output and expected output.
 * The test names should help you to explain what is being tested. You can also see the content
 * of the tests, for the format of the input, see <code>read</code> method.
 * 
 * The output format is described in the comment of <code>output</code> method.
 */

/**
 * This import policy applies to all programming assignments and the exam.
 * 
 * You are forbidden to add any other import statements. Make sure that you do not leave any
 * imports commented out, or that your IDE does not add any automatically. You can recognize
 * allowed imports by the <code>// allowed import</code> comment.
 * Calls by the fully qualified name are treated as imports as well.
 * You can use data structures we provide to you by imports, by our definitions (even
 * implicitly used classes of packages, such as methods of Object those are inherited with
 * every class declaration, Array, or String), or data structures that you write on your own.
 * Usage of common arrays (<code>type[] variable</code>) is not restricted.
 * 
 * Note that Judge is not enforcing this policy. So if you violate this policy, you may lose the
 * points even if Judge accepts your submission.
 * 
 * You may use the following imports without restrictions.
 */
import java.util.StringTokenizer; // allowed import
import java.io.BufferedReader; // allowed import
import java.io.IOException; // allowed import
import java.io.InputStream; // allowed import
import java.io.InputStreamReader; // allowed import
import java.io.PrintStream; // allowed import
import java.util.Arrays; // allowed import
import java.util.Comparator; // allowed import
import java.lang.Comparable; // allowed import
import java.util.List; // allowed import
import java.util.ArrayList; // allowed import
import java.util.LinkedList; // allowed import
import java.util.ListIterator; // allowed import
import java.lang.Math; // allowed import
import java.lang.Integer; // allowed import
import java.lang.String; // allowed import

class Main {
	public static void main(String[] args) {
		ReadAndWrite rw = new ReadAndWrite();
		rw.readAndSolve(System.in, System.out);
	}
}

class Trie {
	class Node {
		Node[] children;
		boolean acc;
		
		public Node() {
			children = new Node[26];
			for (int i = 0; i < 26; i++) {
				children[i] = null;
			}
			acc = false;
		}
	}
	
	String[] dict;
	Node root;
	
	ArrayList<Node> positions;
	
	public Trie(String[] d) {
		dict = d;
		positions = new ArrayList<Node>();
		root = new Node();
	}
	
	void buildTrie() {
		for (int i = 0; i < dict.length; i++) {
			addWord(dict[i]);
		}
	}
	
	void addWord(String word) {
		Node curr = root;
		for (byte c : word.getBytes()) {
			int p = c - 'a';
			if (curr.children[p] != null) {
				curr = curr.children[p];
			} else {
				Node next = new Node();
				curr.children[p] = next;
				curr = next;
			}
		}
		curr.acc = true;
	}
	
	int run(String str) {
		buildTrie();
		positions.add(root);
		int max = 0;
		
		for (int i = 0; i < str.length(); i++) {
			if (positions.isEmpty()) {
				return max;
			}
			
			char c = str.charAt(i);
			
			boolean addNew = false;
			
			for (final ListIterator<Node> it = positions.listIterator(); it.hasNext();) {
				Node child = it.next().children[c-'a'];
				if (child == null) {
					it.remove();
					continue;
				}
				if (child.acc) {
					addNew = true;
					max = i+1;
				}
				it.set(child);
			}
			
			if (addNew) {
				positions.add(root);
			}
		}
		
		return max;
	}
}

class StringSplitter {
	// all string contain only characters a-z
	String[] dictionary; // list of words for that you search in <code>str</code>
	// you can define other classes and new arguments
	Trie t;
	
	/**
	 * Class StringSplitter is initialized once (dictionary is remaining the same)
	 */
	public StringSplitter(String[] d) {
		dictionary = d;
		t = new Trie(dictionary);
	}
	
	/**
	 * TODO: Return the maximal index, up to which the input string
	 * <code>str</code> can be decomposed into words of <code>dictionary</code>.
	 * 
	 * To solve the problem, you will need the following <code>String s</code> method:
	 * <code>s.substring(fromIdx, toIdx)</code> has runtime in O(toIdx - fromIdx)
	 * It extracts the substring from index fromIdx to index toIdx-1 from s.
	 * Mind the index shift, i.e., the index toIdx is not part of the result.
	 * 
	 * The following <code>String s</code> methods are not necessary, but might also be useful:
	 * <code>s.contains(word)</code> has runtime in O(|s| + |word|)
	 * <code>s.indexOf(word)</code> is in O(|s| + |word|)
	 * <code>s.indexOf(word, fromIdx)</code> is in O(|s|-fromIdx + |word|)
	 * <code>s.regionMatches(fromIdx, word, wordIdx, length)</code> is in O(length)
	 * Do to not use <code>s.regionMatches(boolean, fromIdx, word, wordIdx, length)</code>
	 * variant of regionMatches, as it is slower and you do not need to check for upper case
	 * characters in both str and dictionary. 
	 * <code>s.startsWith(word)</code> is in O(min(|s|, |word|))
	 *  
	 * If you need to transform char c = 'a' to 0, and 'z' to 26, you can use
	 * <code>c-97</code> or <code>c-'a'</code>
	 * 
	 * @return The maximal position up to which the string can be split.
	 * Examples:
	 * str = "aabb", dictionary = {"b", "c"} => output is 0
	 * str = "aabb", dictionary = {"a", "c"} => output is 2
	 * str = "aabb", dictionary = {"a", "b"} => output is 4
	 * str = "aabb", dictionary = {"a", "abb"} => output is 4
	 * 
	 * 16 points
	 * 
	 * Test sets:
	 * "unique" the input dictionary is composed of words with unique last character.
	 *          2 points for this task
	 * "small" is a general case, but the time limit is generous.
	 *          3 points for this task
	 * "large" is a general case, but your solution has to be in O(|s|*|D|).
	 *          9 points for this task
	 * "extra_large" is a CHALLENGE. The input is the same as "large",
	 *          but the runtime has to be even faster.
	 *          2 points for this task
	 */
	int solve2(String str) {
		int ml = str.length();
		boolean[] dp = new boolean[ml+1];
		
		for (int i = 1; i < dp.length; i++) {
			dp[i] = false;
		}
		
		dp[0] = true;
		int ret = 0;
		
		for (int i = 0; i < dp.length; i++) {
			if (dp[i]) {
				for (int j = 0; j < dictionary.length; j++) {
					int l = dictionary[j].length();
					
					if (i+l <= ml && str.substring(i, i+l).equals(dictionary[j])) {
						dp[i+l] = true;
						ret = Math.max(ret, i+l);
					}
				}
			}
		}
		return ret;
	}
	
	int solve(String str) {
		return t.run(str);
	}
}

///////////////////////////////////////////////////////////////////////
// DO NOT MODIFY THE FOLLOWING CODE, YOU CAN COMPLETELY IGNORE IT
// WE MAY USE LANGUAGE CONSTRUCTS THAT YOU MAY HAVE NOT SEEN SO FAR
///////////////////////////////////////////////////////////////////////

class ReadAndWrite {
	/**
	 * Parses the input:
	 * <number of words>
	 * <space separated words>
	 * 
	 * <number of test strings>
	 * <newline separated strings>
	 */
	void readAndSolve(InputStream in, PrintStream out) {
		FastReader s = new FastReader(in);

		int dictLen = s.nextInt();
		String[] dict = new String[dictLen];
		for (int i = 0; i < dictLen; i++) {
			dict[i] = s.next();
		}
		StringSplitter sc = new StringSplitter(dict);

		int nStrings = s.nextInt();
		for (int i = 0; i < nStrings; i++) {
			String str = s.next();
			out.println(sc.solve(str));
		}
	}
}

/**
 * Ignore this class please. It is used for input parsing. Source:
 * https://www.geeksforgeeks.org/fast-io-in-java-in-competitive-programming/
 */
class FastReader {
	BufferedReader br;
	StringTokenizer st;

	FastReader(InputStream in) {
		br = new BufferedReader(new InputStreamReader(in));
	}

	String next() {
		while (st == null || !st.hasMoreElements()) {
			try {
				st = new StringTokenizer(br.readLine());
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return st.nextToken();
	}

	int nextInt() {
		return Integer.parseInt(next());
	}

	char nextChar() {
		return next().charAt(0);
	}

	String nextLine() {
		String str = "";
		try {
			str = br.readLine();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return str;
	}
}
